﻿using System;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.Text.RegularExpressions;
using System.Linq;
using System.Net;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Xml;
using System.Net.Mail;
using System.IO;
using System.Text;

using Ektron.Cms;
using Ektron.Cms.Common;
using Ektron.Cms.Instrumentation;
using Ektron.Cms.Extensibility;
using Ektron.Cms.Extensibility.Content;
using Ektron.Cms.Instrumentation;

namespace Cms.Extensions.Templates
{
    public class BlogEventStrategyTemplate : BlogStrategy
    {
        #region STRATEGY EVENTS
        /// <summary>
        /// this event fires when a user enters a blog post on a blog
        /// </summary>
        /// <param name="data">the blog post data</param>
        /// <param name="args">additional data from the bases extension method</param>
        public override void OnAfterAddBlogPost(Ektron.Cms.ContentData data, Ektron.Cms.Extensibility.CmsEventArgs args)
        {
            Ektron.Cms.EmailMessageData msg = GetDefaultSubscriptonMessage(EkEnumeration.EmailMessageTypes.BlogPost);
            System.Collections.Specialized.ListDictionary replacements = GetTokenReplacements(EkEnumeration.EmailMessageTypes.BlogPost, data);
            List<UserData> subscribers = GetSubscriberList(EkEnumeration.EmailMessageTypes.BlogPost, data.Id, args);
            SendSubscriberEmail(EkEnumeration.EmailMessageTypes.BlogPost, msg, replacements, subscribers);
        }
        /// <summary>
        /// this event fires when a comment is made to a blog post
        /// </summary>
        /// <param name="comment">this is the comment data that is being posted</param>
        /// <param name="args">this is additional data from the base extension method</param>
        public override void OnAfterAddBlogComment(Ektron.Cms.BlogComment comment, Ektron.Cms.Extensibility.CmsEventArgs args)
        {
            Ektron.Cms.EmailMessageData msg = GetDefaultSubscriptonMessage(EkEnumeration.EmailMessageTypes.BlogComment);
            System.Collections.Specialized.ListDictionary replacements = GetTokenReplacements(EkEnumeration.EmailMessageTypes.BlogComment, comment);
            List<UserData> subscribers = GetSubscriberList(EkEnumeration.EmailMessageTypes.BlogComment, comment.PostID, args);
            SendSubscriberEmail(EkEnumeration.EmailMessageTypes.BlogComment, msg, replacements, subscribers);
        }

        #endregion

        #region HELPER FUNCTIONS
        /// <summary>
        /// This method checks if the user is a subscriber to the blog or a blog post
        /// </summary>
        /// <param name="oType"> the type of subscription we are checking against</param>
        /// <param name="Id"> the id of the post or the blog</param>
        /// <param name="userId">the user that we are checking subscription on</param>
        /// <returns>a "true" or "false" on whether this is a subscribed user.</returns>
        private string IsSubscriber(EkEnumeration.CMSObjectTypes oType, long Id, long userId)
        {
            try
            {
                Ektron.Cms.ContentAPI _contentAPI = new Ektron.Cms.ContentAPI();
                var wType = _contentAPI.GetWatch(Id, oType, userId, -1);
                return wType.DoesExist.ToString().ToLower();

            }
            catch { return "false"; }
        }
        /// <summary>
        /// this method will replace tokens in a message with appropraite values for that message
        /// </summary>
        /// <param name="template">the message that includes the tokens and text</param>
        /// <param name="replacements">the dictionary of replacement values for the token parser</param>
        /// <param name="mt">the type of the message</param>
        /// <returns>returns the message with the tokens replaced with desired values.</returns>
        static string ReplaceTokens(string template, ListDictionary replacements, Ektron.Cms.Common.EkEnumeration.EmailMessageTypes mt)
        {
            // parse the message into an array of tokens
            Regex regex = null;
            switch (mt)
            {
                case EkEnumeration.EmailMessageTypes.BlogComment:
                    regex = new Regex("(@BlogComment.Id@)|(@BlogComment.Message@)|(@appBlogId@)|(@appBlogName@)");
                    break;
                case EkEnumeration.EmailMessageTypes.BlogPost:
                    regex = new Regex("(@System.GroupProfileTemplate@)|(@System.UserProfileTemplate@)|(@System.SiteUrl@)");
                    break;
                default:
                    break;
            }
            string[] tokens = regex.Split(template);

            // the new message from the tokens
            var sb = new StringBuilder((int)((double)template.Length * 1.1));
            foreach (string token in tokens)
                sb.Append(replacements.Contains(token) ? replacements[token] : token);

            return sb.ToString();
        }
        /// <summary>
        /// checks if the email address is a valid format
        /// </summary>
        /// <param name="email">the email address string</param>
        /// <returns>true or false</returns>
        private bool IsValidEmail(string email)
        {
            try
            {
                var addr = new System.Net.Mail.MailAddress(email);
                return true;
            }
            catch
            {
                return false;
            }
        }
        /// <summary>
        /// this method fetches the default message for the email
        /// </summary>
        /// <param name="mt">the type of message that is being fired by the strategy</param>
        /// <returns>the actual message with the tokens the user has configured</returns>
        private EmailMessageData GetDefaultSubscriptonMessage(EkEnumeration.EmailMessageTypes mt)
        {
            Ektron.Cms.ContentAPI capi = new Ektron.Cms.ContentAPI();
            Ektron.Cms.EmailMessageData msg = new EmailMessageData();

            // At one point I was fetching the data from the configured information that is stored in the DB.
            // However if the Blog message is configured it will fire off the bogus message as well.
            // Decision to move the actual contents of message here.
            // Language support will be needed for multiple languages here.
            //Ektron.Cms.EmailMessageData[] msg = capi.GetDefaultSubscriptionMessageForType(mt);

            switch (mt)
            {
                case EkEnumeration.EmailMessageTypes.BlogComment:
                    msg.Subject = "New Blog Comment Notification";
                    msg.Text = "Tokens: Comment Id:@BlogComment.Id@, Comment Message:@BlogComment.Message@, Blog Id:@appBlogId@, Blog Name:@appBlogName@";
                    break;
                case EkEnumeration.EmailMessageTypes.BlogPost:
                    msg.Subject = "New Blog Post Notification";
                    msg.Text = "Tokens: Site:@System.SiteUrl@, Group Profile:@System.GroupProfileTemplate@, User Profile:@System.UserProfileTemplate@";
                    break;
                default:
                    break;
            }
            return msg;
        }
        /// <summary>
        /// get the list of tokens and the replacement values for use by the token parser
        /// </summary>
        /// <param name="mt">the message type that is being fired by the strategy</param>
        /// <param name="obj">this can be either a post or a comment which are both relevant in this strategy</param>
        /// <returns>the list of mapped tokens</returns>
        private ListDictionary GetTokenReplacements(EkEnumeration.EmailMessageTypes mt, object obj)
        {
            System.Collections.Specialized.ListDictionary replacements = new System.Collections.Specialized.ListDictionary();

            switch (mt)
            {
                case EkEnumeration.EmailMessageTypes.BlogComment:
                    BlogComment bc = (BlogComment)obj;
                    Ektron.Cms.API.Content.Blog bapi = new Ektron.Cms.API.Content.Blog();
                    Ektron.Cms.BlogData blog = bapi.GetBlogDataForPost(bc.PostID);
                    replacements.Add("@BlogComment.Id@", bc.ID.ToString());
                    replacements.Add("@BlogComment.Message@", bc.Message);
                    replacements.Add("@appBlogId@", blog.BlogID.ToString());
                    replacements.Add("@appBlogName@", blog.Name);
                    // continue adding token replacements
                    break;
                case EkEnumeration.EmailMessageTypes.BlogPost:
                    string siteURL = HttpContext.Current.Server.HtmlDecode(HttpContext.Current.Request.Url.AbsoluteUri).ToString();
                    replacements.Add("@System.SiteUrl@", siteURL);
                    replacements.Add("@System.GroupProfileTemplate@", "groupprofile.aspx");
                    replacements.Add("@System.UserProfileTemplate@", "userprofile.aspx");
                    // continue adding token replacements
                    break;
                default:
                    break;
            }
            return replacements;
        }
        /// <summary>
        /// this get the list of subscribers that will get the emails sent to them
        /// </summary>
        /// <param name="mt">this is the message type</param>
        /// <param name="id">this is the id of the blog or post</param>
        /// <param name="args">any additional data that is served by the base extension method</param>
        /// <returns>a list of subscribed users</returns>
        private List<UserData> GetSubscriberList(EkEnumeration.EmailMessageTypes mt, long id, Ektron.Cms.Extensibility.CmsEventArgs args)
        {
            // Get the user list for the subscribers
            Ektron.Cms.DataIO.EkUserRW userIO = new Ektron.Cms.DataIO.EkUserRW(args.RequestInformation);
            List<Ektron.Cms.UserData> userList = new List<UserData>();
            List<Ektron.Cms.UserData> subscribers = new List<UserData>();

            Ektron.Cms.API.Content.Blog bapi = new Ektron.Cms.API.Content.Blog();
            Ektron.Cms.BlogData blog = bapi.GetBlogDataForPost(id);

            /* Get a list of users that are registered and have edit permission to check for subscribers */
            userList = userIO.GetUserListWithPermission(id, "content", "edit", true);


            foreach (Ektron.Cms.UserData item in userList)
            {
                switch (mt)
                {
                    case EkEnumeration.EmailMessageTypes.BlogComment:
                        if ((IsSubscriber(EkEnumeration.CMSObjectTypes.BlogPost, id, item.Id) != "false") || (IsSubscriber(EkEnumeration.CMSObjectTypes.Blog, blog.BlogID, item.Id) != "false"))
                            subscribers.Add(item);
                        break;
                    case EkEnumeration.EmailMessageTypes.BlogPost:
                        if (IsSubscriber(EkEnumeration.CMSObjectTypes.Blog, blog.BlogID, item.Id) != "false")
                            subscribers.Add(item);
                        break;
                    default:
                        break;
                }
            }
            return subscribers;
        }
        /// <summary>
        /// this sends an email to subscribed users for the event in question.
        /// </summary>
        /// <param name="mt">the message type</param>
        /// <param name="msg">the message tied to the event</param>
        /// <param name="replacements">the list of replacement values for tokens</param>
        /// <param name="subscribers">the list of subscribed users to receive email</param>
        private void SendSubscriberEmail(EkEnumeration.EmailMessageTypes mt, Ektron.Cms.EmailMessageData msg, ListDictionary replacements, List<UserData> subscribers)
        {
            if (subscribers.Count > 0)
            {
                Ektron.Cms.ContentAPI capi = new Ektron.Cms.ContentAPI();
                System.Net.Mail.MailMessage mm = new System.Net.Mail.MailMessage();
                mm.From = new System.Net.Mail.MailAddress(capi.RequestInformationRef.SystemEmail);

                foreach (Ektron.Cms.UserData item in subscribers)
                {
                    if (IsValidEmail(item.Email))
                        mm.To.Add(item.Email);
                }

                if (mm.To.Count > 0)
                {
                    mm.Subject = ReplaceTokens(msg.Subject, replacements, mt);
                    mm.Body = ReplaceTokens(msg.Text, replacements, mt);
                    mm.IsBodyHtml = true;

                    System.Net.Mail.SmtpClient client = new System.Net.Mail.SmtpClient(capi.RequestInformationRef.SMTPServerName, capi.RequestInformationRef.SMTPPort);
                    client.Send(mm);
                }
            }
        }

        #endregion
    }
}
